pub struct BackendProcessInfo {
pub frames: usize,
pub process_timestamp: Option<Instant>,
pub duration_since_stream_start: Duration,
pub input_stream_status: StreamStatus,
pub output_stream_status: StreamStatus,
pub dropped_frames: u32,
pub process_to_playback_delay: Option<Duration>,
}Fields
frames: usizeprocess_timestamp: Option<Instant>duration_since_stream_start: Durationinput_stream_status: StreamStatusoutput_stream_status: StreamStatusdropped_frames: u32process_to_playback_delay: Option<Duration>The estimated time between when this process loop was called and when the data will be delivered to the output device for playback.
If the audio backend does not provide this information, then set
this to None.