pub struct SimpleDeviceConfig {
pub device: Option<String>,
pub channels: Option<usize>,
}Expand description
The configuration of an input/output device for a SimpleStreamConfig
Fields
device: Option<String>The ID of the device to use. (The ID from DeviceInfoSimple::id.)
Set to None to use the default device.
By default this is set to None.
channels: Option<usize>The number of channels to use. The backend may end up using a different channel count if the given channel count is not supported.
Set to None to use the default number of channels for the device.
By default this is set to None.