pub struct SimpleStreamConfig {
pub output: SimpleDeviceConfig,
pub input: Option<SimpleDeviceConfig>,
pub desired_block_frames: Option<u32>,
pub desired_sample_rate: Option<u32>,
}Expand description
An easy-to-use, backend-agnostic configuration for an audio stream
Fields
output: SimpleDeviceConfigThe configuration of the output audio device
input: Option<SimpleDeviceConfig>The configuration of the input audio device
Set to None to not connect an input audio device.
By default this is set to None.
desired_block_frames: Option<u32>The block size (latency) to use. Set to None to use the device’s default
block size. The backend may end up using a different block size if the given
block size is not supported.
By default this is set to Some(1024), which should be good enough for most
games.
If your application is not a game and doesn’t need low latency playback,
then prefer to set this to None to reduce system resources.
desired_sample_rate: Option<u32>The sample rate to use. The backend may end up using a different sample rate if the given sample rate is not supported.
Set to None to use the device’s default sample rate.
By default this is set to None.