Module error

Module error 

Enums

ActivateError
An error occurred while attempting to activate a FirewheelContext.
AddEdgeError
An error occurred while attempting to add an edge to the graph.
CompileGraphError
An error occurred while attempting to compile the audio graph into a schedule.
DeactivateError
An error occurred while deactivate a FirewheelContext.
ModifyGraphError
RemoveNodeError
An error while removing a node in FirewheelContext.
UpdateError
An error occurred while updating a FirewheelContext.