pub enum ModifyGraphError {
NodeError(NodeError),
RemoveNodeError(RemoveNodeError),
AddEdgeError(AddEdgeError),
UpdateError(UpdateError),
CompileGraphError(CompileGraphError),
}Variants
NodeError(NodeError)
An error occured while adding a new node to the graph.
RemoveNodeError(RemoveNodeError)
An error occured while removing a node from the graph.
AddEdgeError(AddEdgeError)
An error occured while adding a new edge to the graph.
UpdateError(UpdateError)
An error occurred while updating a FirewheelContext.
CompileGraphError(CompileGraphError)
An error while trying to compile the graph, i.e. a cycle was detected.