BufferOutOfSpaceMode

Enum BufferOutOfSpaceMode 

pub enum BufferOutOfSpaceMode {
    AllocateOnAudioThread,
    Panic,
    DropEvents,
}
How to handle event buffers on the audio thread running out of space.

Variants



AllocateOnAudioThread

If an event buffer on the audio thread ran out of space to fit new events, reallocate on the audio thread to fit the new items. If this happens, it may cause underruns (audio glitches), and a warning will be logged.



Panic

If an event buffer on the audio thread ran out of space to fit new events, then panic.



DropEvents

If an event buffer on the audio thread ran out of space to fit new events, drop those events to avoid allocating on the audio thread. If this happens, a warning will be logged.

(Not generally recommended, but the option is here if you want it.)