pub struct ProfilingData {
pub version: u64,
pub overall_cpu_usage: f64,
pub engine_bookkeeping_cpu_usage: Option<f64>,
pub nodes: Vec<NodeProfileData>,
}Expand description
Performance profiling information of a Firewheel Processor.
Fields
version: u64The number of times the profiling data has been updated.
overall_cpu_usage: f64The overall CPU usage of the entire Firewheel Processor’s process method.
A value of 0.0 means 0% CPU usage, a value of 1.0 means 100%
CPU usage of the total alloted time, and values above 1.0 means an
underrun has occurred.
The value is the maximum value that has occurred since the last time
the profiling information was fetched with
FirewheelContext::profiling_data().
engine_bookkeeping_cpu_usage: Option<f64>The CPU usage of engine bookkeeping operations such as message handling, event sorting, event searching, and final output processing.
A value of 0.0 means 0% CPU usage, and a value of 1.0 means 100%
CPU usage of the total allotted time. (If instead you want the fraction
of time spent relative to the total time spent in the Firewheel process
method, it can be found with bookkeeping_cpu_usage / overall_cpu_usage.)
The value is the maximum value that has occurred since the last time
the profiling information was fetched with
FirewheelContext::profiling_data().
If FirewheelFlags::profile_engine_bookkeeping
is set to false (which it is by default), then this will be None.
nodes: Vec<NodeProfileData>The profiling information of individual nodes.
The order in which nodes appear is not defined.
This may be empty if FirewheelFlags::profile_nodes
is set to false (which it is by default) or if there was an error
with running out of buffer space.