pub struct ActivateInfo {
pub sample_rate: NonZeroU32,
pub max_block_frames: NonZeroU32,
pub num_stream_in_channels: u32,
pub num_stream_out_channels: u32,
pub input_to_output_latency_seconds: f64,
}Expand description
Information about the running audio stream.
Fields
sample_rate: NonZeroU32The sample rate of the audio stream.
max_block_frames: NonZeroU32The maximum number of frames that can appear in a single process cyle.
num_stream_in_channels: u32The number of input audio channels in the stream.
num_stream_out_channels: u32The number of output audio channels in the stream.
input_to_output_latency_seconds: f64The latency of the input to output stream in seconds.