pub struct ContextQueue<'a> {
pub context: &'a mut FirewheelContext,
/* private fields */
}Expand description
An event queue acquired from FirewheelContext::event_queue.
This can help reduce event queue allocations when you have direct access to the context.
// Get a queue that will send events directly to the provided node.
let mut queue = context.event_queue(node_id);
// Perform diffing using this queue.
params.diff(baseline, PathBuilder::default(), &mut queue);Fields
context: &'a mut FirewheelContextImplementations
Sourceimpl ContextQueue<'_>
impl ContextQueue<'_>
Sourcepub fn push_bypassed(&mut self, bypassed: bool)
pub fn push_bypassed(&mut self, bypassed: bool)
Send an event to set the bypass state of the node.
Sourceimpl ContextQueue<'_>
impl ContextQueue<'_>
pub fn time(&self) -> Option<EventInstant>