pub struct FirewheelFlags {
pub hard_clip_outputs: bool,
pub detect_clipping_on_output: bool,
pub validate_output_is_finite: bool,
pub force_clear_buffers: bool,
pub profile_engine_bookkeeping: bool,
pub profile_nodes: bool,
}Expand description
Configuration flags for a FirewheelContext
Unlike FirewheelConfig, these flags can be changed after the context has
been created.
Fields
hard_clip_outputs: boolHard clip all samples in the final output buffer to the range [-1.0, 1.0].
This usually isn’t necessary since the OS itself generally hard clips the output, but it is here if you need it.
By default this is set to false.
detect_clipping_on_output: boolDetect when a sample in the final output buffer falls outside the range
[-1.0, 1.0]. If a sample falls outside this range, then
FirewheelContext::clipping_occurred will return true.
This check takes place before hard clipping if the
FirewheelFlags::hard_clip_outputs is set to true.
By default this is set to false.
validate_output_is_finite: boolValidate that all samples in the final output buffer are a valid finite
number. If a non finite number is detected, then the sample will will be
set to 0.0 and an error is logged.
By default this is set to false.
force_clear_buffers: boolForce all of a node’s output buffers to be cleared before processing. This shouldn’t be necessary, but it can be used to debug nodes that are misusing the silence flag feature.
By default this is set to false.
profile_engine_bookkeeping: boolEnables performance profiling for engine bookkeeping operations on the audio thread such as message handling, event sorting, event searching, and final output processing.
By default this is set to false.
profile_nodes: boolEnable per-node performance profiling.
This has no effect when the node_profiling feature is disabled.
By default this is set to false.