Enums
- Activate
Error - An error occurred while attempting to activate a
FirewheelContext. - AddEdge
Error - An error occurred while attempting to add an edge to the graph.
- Compile
Graph Error - An error occurred while attempting to compile the audio graph into a schedule.
- Deactivate
Error - An error occurred while deactivate a
FirewheelContext. - Modify
Graph Error - Remove
Node Error - An error while removing a node in
FirewheelContext. - Update
Error - An error occurred while updating a
FirewheelContext.